@@ -16,15 +16,6 @@
 #
 # Author: Francois Boulogne <fboulogne at sciunto dot org>, 2012
 
-
-class TARGETError(Exception):
-    """
-    Exception for target validity
-    :param message: Message
-    """
-    def __init__(self, message=''):
-        Exception.__init__(self, message)
-
 import os
 import re
 #import psutil
@@ -37,15 +28,23 @@ import logging
 from contextlib import closing
 
 
+class TARGETError(Exception):
+    """
+    Exception for target validity
+    :param message: Message
+    """
+    def __init__(self, message=''):
+        Exception.__init__(self, message)
+
+
 class Target:
     """
     A target is a source or a destination.
     Both of them can be a local directory
     or a distant one though SSH
     """
-    
     def __init__(self, target):
-        self.logger = logging.getLogger('Vitalus.Target') 
+        self.logger = logging.getLogger('Vitalus.Target')
         self.logger.debug("Read target %s", target)
         self.target = target
         self.ttype = self._detect_target_type()
@@ -55,14 +54,23 @@ class Target:
         elif self.is_ssh():
             self.login, self.path = target.split(':')
 
-
     def is_dir(self):
+        """
+        Check if the target is a directory
+
+        :returns: boolean
+        """
         if self.ttype == 'DIR':
             return True
         else:
             return False
 
     def is_ssh(self):
+        """
+        Check if the target is a 'SSH' host
+
+        :returns: boolean
+        """
         if self.ttype == 'SSH':
             return True
         else:
@@ -73,18 +81,18 @@ class Target:
         Return the target type
         SSH if matches name@domaine.tld:dir
         DIR if it's a directory
-       
+
         :param target: Target (source or destination)
-        :raises TARGETError: weird target type 
+        :raises TARGETError: weird target type
         """
-        if re.match('[a-zA-Z0-9+_\-\.]+@[0-9a-zA-Z][.-0-9a-zA-Z]*.[a-zA-Z]+\:.*', self.target):
+        #if re.match('[a-zA-Z0-9+_\-\.]+@[0-9a-zA-Z][.-0-9a-zA-Z]*.[a-zA-Z]+\:.*', self.target):
+        if re.match('[a-zA-Z0-9+_\-\.]+@[a-zA-Z0-9+_\-\.]+\:.*', self.target):
             #ssh
             self.logger.debug("The target %s looks like SSH", self.target)
             return 'SSH'
         else:
             if not os.path.exists(self.target):
-                self.logger.warn("The target %s does not exist", self.target)
-                self.logger.info('Aborting...')
+                self.logger.warning("The target %s does not exist", self.target)
                 raise TARGETError("Target %s does not exist" % self.target)
             else:
                 self.logger.debug("The target %s looks like DIR", self.target)
@@ -113,10 +121,9 @@ class Job:
     Source and destination path can be either real path
     or a ssh login joined to the path by a : caracter.
     """
-    #TODO signal...
 
     def __init__(self, log_dir, name, source, destination, period, snapshot, duration, keep, filter):
-       
+
         self.name = name
         self.source = Target(source)
         self.destination = Target(destination)
@@ -130,30 +137,20 @@ class Job:
         self.current_date = self.now.strftime("%Y-%m-%d_%Hh%Mm%Ss")
 
         self.backup_log_dir = log_dir
-        
-        self.logger = logging.getLogger('Vitalus.Job') #TODO add the job name in the format
-        
+
+        self.logger = logging.getLogger('Vitalus.Job')
+
         #Logs specific to the rsync job
         job_log = os.path.join(self.backup_log_dir, self.name + '.log')
         self.job_logger = logging.getLogger(self.name)
         log_rotator = logging.handlers.TimedRotatingFileHandler(job_log, when='midnight', interval=1, backupCount=30, encoding=None, delay=False, utc=False)
         self.job_logger.addHandler(log_rotator)
         self.job_logger.setLevel(logging.INFO)
-        
+
         #Set previous and current backup paths
         self.previous_backup_path = None
         self.current_backup_path = None
 
-        last_date = self._get_last_backup()
-        self.current_backup_path = os.path.join(self.destination.path , self.name, str(self.current_date))
-        if last_date is None:
-            #It means that this is the first backup.
-            self.previous_backup_path = None
-        else:
-            self.previous_backup_path = os.path.join(self.destination.path, self.name, str(last_date))
-
-        self.logger.debug("Previous backup path: %s", self.previous_backup_path)
-        self.logger.debug("Current backup path: %s", self.current_backup_path)
 
 
 #    def _check_disk_usage(self):
@@ -164,7 +161,7 @@ class Job:
 #        if self.destination.is_dir():
 #            #TODO, change the criterion
 #            pass
-#            #if psutil.disk_usage(self.destination)[2] < utils.get_folder_size(self.source): 
+#            #if psutil.disk_usage(self.destination)[2] < utils.get_folder_size(self.source):
 #            #    self.logger.critical("Low disk space: %s", self.destination)
 #            #    raise TARGETError('Low disk space on %s' % self.destination)
 #        elif self.destination.is_ssh():
@@ -173,7 +170,7 @@ class Job:
 
     def _set_lastbackup_time(self):
         """
-        Set the last backup (labeled name) time 
+        Set the last backup (labeled name) time
         """
         self.logger.debug('Set lastbackup time')
         with closing(shelve.open(os.path.join(self.backup_log_dir, 'time.db'))) as timebase:
@@ -190,13 +187,13 @@ class Job:
         self.logger.debug("Check time between backups for %s", self.name)
         with closing(shelve.open(os.path.join(self.backup_log_dir, 'time.db'))) as timebase:
             try:
-                last = timebase[self.name] 
+                last = timebase[self.name]
             except KeyError:
                 #Not yet stored
                 #Run the first backup
                 self.logger.debug("%s: first backup", self.name)
                 return True
-       
+
         #Calculate the difference
         self.logger.debug("now= %s", datetime.datetime.now())
         self.logger.debug("last= %s", last)
@@ -211,7 +208,7 @@ class Job:
             self.logger.debug("%s does not need backup", self.name)
             return False
 
-    def _delete_old_files(self, days=10, keep=10): 
+    def _delete_old_files(self, days=10, keep=10):
         """
         Delete old archives in the destination
 
@@ -221,64 +218,57 @@ class Job:
         #TODO : review logs
 
         path = os.path.join(self.destination.path, self.name)
-        ####if self.terminate: return
-        
+
         if self.destination.is_dir():
-            filenames = [os.path.join(path, el) for el in os.listdir(path) ]
+            #filenames = [os.path.join(path, el) for el in os.listdir(path) ]
+            filenames = os.listdir(path)
         elif self.destination.is_ssh():
             command = ['ssh', '-t', self.destination.login, 'ls', '-1', path]
             self.logger.debug('SSH ls command: ' + str(command))
             process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
             stdout, stderr = process.communicate()
             filenames = stdout.decode()
-            filenames = filenames.split('\r\n')
-            filenames = [x for x in filenames if x!='']
+            filenames = filenames.split('\n')
+            filenames = [x.strip('\r') for x in filenames if x!='']
         else:
             return
 
-        if self.destination.is_ssh():
-            #Not yet implemented!
-            return
+        self.logger.debug("file_list %s ", filenames)
 
-        #delete first the older one
-        to_delete = []
-        filenames.sort() #newer first
-        nb_archives = len(filenames)
-        for filepath in enumerate(filenames):
-            #keep a sufficient number of archives
-            if (nb_archives - filepath[0]) <= keep:
-                self.logger.debug('nb archives %s, keep %s, archive %s' % (nb_archives, keep, filepath[0]))
-                continue
-            #Are they too old?
-            file_date = datetime.datetime.fromtimestamp(os.path.getmtime(filepath[1]))
-            if datetime.datetime.now() - file_date > datetime.timedelta(days=days): 
+        to_delete = utils.get_older_files(filenames, days, keep)
 
-                to_delete.append(filepath[1])
-                self.logger.info("Remove backup %s", filepath[1])
-            else:
-                self.logger.debug("Keep backup %s", filepath[1])
-                continue
+        self.logger.debug("to delete %s ", to_delete)
 
-        for element in to_delete:
-            if self.destination.is_dir():
-                print(element) #FIXME
+        if self.destination.is_dir():
+            for element in to_delete:
+                self.logger.debug("Remove backup %s", element)
                 try:
-                    shutil.rmtree(element)
+                    shutil.rmtree(os.path.join(path, element))
                 except OSError:
                     self.logger.error("Impossible to delete %s (symlink?)", element)
-            elif self.destination.is_ssh():
-                print(element) #FIXME
+        elif self.destination.is_ssh():
+            filepaths = [os.path.join(path, element) for element in to_delete]
+            if filepaths != []:
+                command = ['ssh', '-t', self.destination.login, 'rm', '-rf']
+                command.extend(filepaths)
+                self.logger.debug('SSH rm command: ' + str(command))
+                process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
+                stdout, stderr = process.communicate()
+
 
     def _get_last_backup(self):
         """
-        Get the last backup in path
+        Get the last backup path
         Return None if not available
+
+        :returns: string
         """
         path = os.path.join(self.destination.path, self.name)
         if self.destination.is_dir():
             if not os.path.isdir(path):
                 return None
-            filenames = [os.path.join(path, el) for el in os.listdir(path)]
+            #filenames = [os.path.join(path, el) for el in os.listdir(path)]
+            filenames = os.listdir(path)
         elif self.destination.is_ssh():
             #First, create at least the target if does not exists
             command = ['ssh', '-t', self.destination.login, 'mkdir', '-p', path]
@@ -293,28 +283,25 @@ class Job:
             process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
             stdout, stderr = process.communicate()
             filenames = stdout.decode()
-            filenames = filenames.split('\r\n')
-            filenames = [x for x in filenames if x!='']
+            filenames = filenames.split('\n')
+            filenames = [x.strip('\r') for x in filenames if x!='']
 
-        filenames.sort()
-        if filenames == []:
-            return None
-        try:
-            return filenames[-1] 
-        except IndexError:
-            return None
+        last = utils.get_last_file(filenames)
+        last = os.path.join(path, last)
+        self.logger.debug('_get_last_backup returns: %s', last)
+        return last
 
-    def _prepare_destination(self): 
+    def _prepare_destination(self):
         """
         Prepare the destination to receive a backup:
         Create dirs
         """
         if self.destination.is_dir():
             if self.snapshot:
-                os.makedirs(self.current_backup_path) #This one does not exist!
+                os.makedirs(self.current_backup_path) # This one does not exist!
             else:
                 if self.previous_backup_path is None:
-                    os.makedirs(self.current_backup_path, exist_ok=True) 
+                    os.makedirs(self.current_backup_path, exist_ok=True)
                 else:
                     #Move dir to set the new date in the path
                     os.rename(self.previous_backup_path, self.current_backup_path)
@@ -325,12 +312,9 @@ class Job:
             process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
             stdout, stderr = process.communicate()
             self.logger.debug('SSH mkdir result: ' + stdout.decode())
-                
-        self.logger.debug("Previous backup path: %s", self.previous_backup_path)
-        self.logger.debug("Current backup path: %s", self.current_backup_path)
 
 
-    def _prepare_rsync_command(self): 
+    def _prepare_rsync_command(self):
         """
         Compose the rsync command
         """
@@ -343,7 +327,7 @@ class Job:
         # stat: file rate stats
         # delete: delete extraneous files from dest dirs
         # delete-excluded: also delete excluded files from dest dirs
-        command.append('-avh') 
+        command.append('-avh')
         command.append('--stats')
         command.append('--delete')
         command.append('--delete-excluded')
@@ -373,12 +357,12 @@ class Job:
             for element in self.filter:
                 command.append('--filter=' + element)
                 self.logger.debug("add filter: %s", element)
-                
+
         self.logger.debug("rsync command: %s", command)
         return command
 
     def _run_command(self, command):
-        """ 
+        """
         Run a command and log stderr+stdout in a dedicated log file.
         :param command: a list, each element is a part of the command line
 
@@ -386,10 +370,6 @@ class Job:
         -------
         command = ['/usr/bin/cp', '-r', '/home', '/tmp']
         """
-
-        #If a signal is received, stop the process
-        #if self.terminate: return
-
         #Run the command
         process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         stdout, stderr = process.communicate()
@@ -406,37 +386,59 @@ class Job:
         """
         Run the job
         """
-        #TODO rewriting and integration
+        self.logger.debug('Start job: %s', self.name)
+        #TODO rewriting and integration:
         #self._check_disk_usage()
+
+        last_date = self._get_last_backup()
+        self.current_backup_path = os.path.join(self.destination.path, self.name, str(self.current_date))
+        if last_date is None:
+            #It means that this is the first backup.
+            self.previous_backup_path = None
+        else:
+            #self.previous_backup_path = os.path.join(self.destination.path, self.name, str(last_date))
+            self.previous_backup_path = last_date
+
+        self.logger.debug("Previous backup path: %s", self.previous_backup_path)
+        self.logger.debug("Current backup path: %s", self.current_backup_path)
+
         if self._check_need_backup():
             self.job_logger.info('='*20 + str(self.now) + '='*20)
+            self.logger.debug('Start Backup: %s', self.name)
             print(self.name)
-            self.logger.info("Backup %s", self.name)
-            #Prepare the destination
+
+            # Prepare the destination
             self._prepare_destination()
-            self.logger.debug("source path %s", self.source.target) 
-            self.logger.debug("destination path %s", self.destination.target) 
+            self.logger.debug("source path %s", self.source.target)
+            self.logger.debug("destination path %s", self.destination.target)
             self.logger.debug("filter path %s", self.filter)
-            #Run rsync
+
+            # Run rsync
             command = self._prepare_rsync_command()
             self._run_command(command)
-            #Job done, update the time in the database
+
+            # Job done, update the time in the database
             self._set_lastbackup_time()
-            #Remove old snapshots
-            #self._delete_old_files(days=self.duration, keep=self.keep)
-            self._delete_old_files(days=0, keep=3)
-            #Create symlink TODO: SSH
-            if self.destination.is_dir() and self.snapshot:
-                #Symlink
-                last = os.path.join(self.destination.path, self.name, 'last')
+
+            # Remove old snapshots
+            self._delete_old_files(days=10, keep=10) # TODO adjustable in API
+
+            # Create symlink
+            last = os.path.join(self.destination.path, self.name, 'last')
+            if self.destination.is_dir():
                 if os.path.islink(last):
                     os.remove(last)
                 os.chdir(os.path.dirname(self.current_backup_path))
                 try:
                     os.symlink(os.path.basename(self.current_backup_path), last)
                 except FileExistsError:
-                    self.logger.warn('The symlink %s could not be created because a file exists', last)
+                    self.logger.warning('The symlink %s could not be created because a file exists', last)
                 except AttributeError:
-                    self.logger.warn('Attribute error for symlink. Job: %s', self.name) 
+                    self.logger.warning('Attribute error for symlink. Job: %s', self.name)
+            elif self.destination.is_ssh():
+                pass
+                #TODO Create symlink
+
+            self.logger.info("Backup %s done", self.name)
 
 
