@@ -16,10 +16,9 @@
 #
 # Author: Francois Boulogne <fboulogne at sciunto dot org>, 2012
 
-#import datetime
 import os
 import psutil
-import signal
+#import signal
 import logging, logging.handlers
 import sys
 
@@ -61,10 +60,8 @@ class Vitalus:
         #Priority 
         self._set_process_low_priority()
 
-        #timebase
-        
         self.logger.info('Vitalus %s starts...' % info.VERSION)
-        signal.signal(signal.SIGTERM, self._signal_handler)
+        #signal.signal(signal.SIGTERM, self._signal_handler)
 
     def set_log_level(self, level='INFO'):
         """
@@ -92,10 +89,10 @@ class Vitalus:
             
 
 
-    def _signal_handler(self, signal, frame):
-        self.logger.warn('Signal received %s', signal)
-        self._set_process_high_priority()
-        self.terminate = True
+#    def _signal_handler(self, signal, frame):
+#        self.logger.warning('Signal received %s', signal)
+#        self._set_process_high_priority()
+#        self.terminate = True
 
     def _create_pidfile(self):
         """ Create a pidfile """
@@ -164,6 +161,11 @@ class Vitalus:
         Note: filter syntax is the same of rsync. See "FILTER RULES" section
         in the rsync man page.
         """
+        if name in self.jobs:
+            self.critical("%s already present in the job list. Job's name should be uniq.", name)
+            return
+
+
         if self.destination:
             period_in_seconds = period * 3600
             self.logger.debug("add job: %s", name) 
@@ -171,8 +173,7 @@ class Vitalus:
                 self.jobs.append(Job(self.backup_log_dir, name, source, 
                     self.destination, period_in_seconds, history, duration, keep, filter))
             except TARGETError:
-                #FIXME: message
-                self.logger.warning('Wrong or unreachable destination')
+                #We abort
                 pass
 
 
@@ -189,7 +190,6 @@ if __name__ == '__main__':
     b = Vitalus()
     b.set_log_level('DEBUG')
     b.set_destination('/tmp/sauvegarde')
-    #TODO Check that job names are uniq
     b.add_job('test', '/home/gnu/tmp/firefox', period=0.0, history=True)
     b.add_job('test2', '/home/gnu/tmp/debian', period=0.0, history=True)
     b.add_job('test3', '/home/gnu/tmp/photos', period=0.001, history=False)
